package processing

import (
	"cloud.o-forge.io/core/oc-lib/dbs/mongo"
	"cloud.o-forge.io/core/oc-lib/models/utils"
)

type ProcessingMongoAccessor struct {
	utils.AbstractAccessor
}

func (pma *ProcessingMongoAccessor) DeleteOne(id string) (utils.DBObject, int, error) {
	return pma.GenericDeleteOne(id, pma)
}

func (pma *ProcessingMongoAccessor) UpdateOne(set utils.DBObject, id string) (utils.DBObject, int, error) {
	return pma.GenericUpdateOne(set, id, pma, &ProcessingResource{})
}

func (pma *ProcessingMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, int, error) {
	return pma.GenericStoreOne(data, pma)
}

func (pma *ProcessingMongoAccessor) CopyOne(data utils.DBObject) (utils.DBObject, int, error) {
	return pma.GenericStoreOne(data, pma)
}

func (pma *ProcessingMongoAccessor) LoadOne(id string) (utils.DBObject, int, error) {

	var processing ProcessingResource

	res_mongo, code, err := mongo.MONGOService.LoadOne(id, pma.GetType())
	if err != nil {
		pma.Logger.Error().Msg("Could not retrieve " + id + " from db. Error: " + err.Error())
		return nil, code, err
	}

	res_mongo.Decode(&processing)

	return &processing, 200, nil
}

func (wfa ProcessingMongoAccessor) LoadAll() ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	res_mongo, code, err := mongo.MONGOService.LoadAll(wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not retrieve any from db. Error: " + err.Error())
		return nil, code, err
	}
	var results []ProcessingResource
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	for _, r := range results {
		objs = append(objs, &r.AbstractResource)
	}
	return objs, 200, nil
}

func (wfa *ProcessingMongoAccessor) Search(word string) ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	res_mongo, code, err := mongo.MONGOService.Search(word, []string{"name", "short_description", "description", "owner", "source_url"}, wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not store to db. Error: " + err.Error())
		return nil, code, err
	}
	var results []ProcessingResource
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	for _, r := range results {
		objs = append(objs, &r.AbstractResource)
	}
	return objs, 200, nil
}
