package storage

import (
	"cloud.o-forge.io/core/oc-lib/dbs/mongo"
	"cloud.o-forge.io/core/oc-lib/models/utils"
)

type StorageMongoAccessor struct {
	utils.AbstractAccessor
}

func (sma *StorageMongoAccessor) DeleteOne(id string) (utils.DBObject, int, error) {
	return sma.GenericDeleteOne(id, sma)
}

func (sma *StorageMongoAccessor) UpdateOne(set utils.DBObject, id string) (utils.DBObject, int, error) {
	return sma.GenericUpdateOne(set, id, sma, &StorageResource{})
}

func (sma *StorageMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, int, error) {
	return sma.GenericStoreOne(data, sma)
}

func (sma *StorageMongoAccessor) CopyOne(data utils.DBObject) (utils.DBObject, int, error) {
	return sma.GenericStoreOne(data, sma)
}

func (sma *StorageMongoAccessor) LoadOne(id string) (utils.DBObject, int, error) {

	var storage StorageResource

	res_mongo, code, err := mongo.MONGOService.LoadOne(id, sma.GetType())
	if err != nil {
		sma.Logger.Error().Msg("Could not retrieve " + id + " from db. Error: " + err.Error())
		return nil, code, err
	}

	res_mongo.Decode(&storage)

	return &storage, 200, nil
}

func (wfa StorageMongoAccessor) LoadAll() ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	res_mongo, code, err := mongo.MONGOService.LoadAll(wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not retrieve any from db. Error: " + err.Error())
		return nil, code, err
	}
	var results []StorageResource
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	for _, r := range results {
		objs = append(objs, &r.AbstractResource)
	}
	return objs, 200, nil
}

func (wfa *StorageMongoAccessor) Search(word string) ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	res_mongo, code, err := mongo.MONGOService.Search(word, []string{"name", "short_description", "description", "owner", "source_url"}, wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not store to db. Error: " + err.Error())
		return nil, code, err
	}
	var results []StorageResource
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	for _, r := range results {
		objs = append(objs, &r.AbstractResource)
	}
	return objs, 200, nil
}
